<?php

/**
 * Carerix Wordpress Plugin
 *
 * @category settings
 * @author Bogdan Ivanov
 * @copyright Copyright (c) 2013 Carerix.com (http://www.carerix.com)
 * @link http://www.carerix.com
 * @since Sep 2, 2013 2:09:59 PM
 * @version $Rev$
 */

use Carerix\Api\Rest\Entity\CRDataNode;
use Carerix\Api\Rest\Entity\CREmployee;
use Carerix\Api\Rest\Entity\CRUser;

/**
 * Apply class
 * Used to ....
 */
class Entity_Form_Apply extends Entity_Form {
	protected $type = WP_Constants::ENTITY_TYPE_FORM_APPLY;
	
	/**
	* @var CRUser
	*/
	static private $parsed_candidate = null;
	

	const FIELD_TYPE_TEXT = 'text';
	const FIELD_TYPE_PASSWORD = 'password';
	const FIELD_TYPE_SYSTEM = 'system';
	const FIELD_TYPE_CHECK = 'check';
	const FIELD_TYPE_GROUP = 'group';
	const FIELD_TYPE_DATE = 'date';
	const FIELD_TYPE_FILE = 'file';
	
	static $FIELDS_TO_DISPLAY = array(
                'hobbies' => array(
                    'type'      => self::FIELD_TYPE_TEXT,
                    'mapsTo'    => 'hobbies'
                ),
		'lastName' => array(
			'type' => self::FIELD_TYPE_TEXT,
			'mapsTo' => 'lastName',
		),
		
		'firstName' => array(
			'type' => self::FIELD_TYPE_TEXT,
			'mapsTo' => 'firstName',
		),
		
		'lastNameAtBirth' => array(
			'type' => self::FIELD_TYPE_TEXT,
			'mapsTo' => 'birthName',
		),
		
		'title' => array(
			'type' => self::FIELD_TYPE_TEXT,
			'mapsTo' => 'title',
		),
		
		'suffix' => array(
			'type' => self::FIELD_TYPE_TEXT,
			'mapsTo' => 'suffix',
		),
		
		'password' => array(
			'type' => self::FIELD_TYPE_PASSWORD,
			'mapsTo' => 'password',
		),
		
		'gender' => array(
			'type' => self::FIELD_TYPE_SYSTEM,
			'mapsTo' => 'genderNode',
			'system_table' => 'Salutation'
		),
		
		'maritalStatus' => array(
			'type' => self::FIELD_TYPE_SYSTEM,
			'mapsTo' => 'maritalStatusNode',
			'system_table' => 'Marital Status',
		),
		
		'birthDate' => array(
			'type' => self::FIELD_TYPE_DATE,
			'mapsTo' => 'birthDate'
		),
		
		'birthPlace' => array(
			'type' => self::FIELD_TYPE_TEXT,
			'mapsTo' => 'birthCity',
		),
		
		'countryOfBirth' => array(
			'type' => self::FIELD_TYPE_SYSTEM,
			'mapsTo' => 'birthCountryNode',
			'system_table' => 'Country',
		),
		
		'nationality' => array(
			'type' => self::FIELD_TYPE_SYSTEM,
			'mapsTo' => 'nationalityNode',
			'system_table' => 'Nationality',
		),
		
		'functionLevel' => array(
			'type' => self::FIELD_TYPE_SYSTEM,
			'mapsTo' => 'Function1Level1Node',
			'system_table' => 'Business line',
		),
		
		'experience' => array(
			'type' => self::FIELD_TYPE_SYSTEM,
			'mapsTo' => 'ExperienceNode',
			'system_table' => 'Experience',
		),
		
		'cv' => array(
			'type' => self::FIELD_TYPE_FILE,
			'mapsTo' => 'cv',
		), // [todo]
		
		'photo' => array(
			'type' => self::FIELD_TYPE_FILE,
			'mapsTo' => 'photo',
		), // [todo]
		
		'coverLetter' => array(
			'type' => self::FIELD_TYPE_FILE,
			'mapsTo' => 'coverLetter',
		), // [todo]
		
		'email' => array(
			'type' => self::FIELD_TYPE_TEXT,
			'mapsTo' => 'EmailAddress',
		),
		
		'homePhone' => array(
			'type' => self::FIELD_TYPE_TEXT,
			'mapsTo' => 'PhoneNumber',
		),
		
		'workPhone' => array(
			'type' => self::FIELD_TYPE_TEXT,
			'mapsTo' => 'PhoneNumberBusiness',
		),
		
                'mobileNumber' => array(
			'type' => self::FIELD_TYPE_TEXT,
			'mapsTo' => 'mobileNumber',
		),
            
                'mobileNumberBusiness' => array(
			'type' => self::FIELD_TYPE_TEXT,
			'mapsTo' => 'mobileNumberBusiness',
		),
            
		'contactInstructions' => array(
			'type' => self::FIELD_TYPE_TEXT,
			'mapsTo' => 'ContactInfo',
		),
		
		'homeAddress' => array(
			'type' => self::FIELD_TYPE_GROUP,
			'items' => array(
				'streetName' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'HomeStreet',
				),
				'streetNumber' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'HomeNumber',
				),
				'suffixNumber' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'HomeNumberSuffix',
				),
				'city' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'city',
				), // [todo] no node to map to
				'postalCode' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'HomePostalCode',
				),
				'country' => array(
					'type' => self::FIELD_TYPE_SYSTEM,
					'mapsTo' => 'HomeCountryNode',
					'system_table' => 'Country',
				),
				'region' => array(
					'type' => self::FIELD_TYPE_SYSTEM,
					'mapsTo' => 'HomeProvinceNode',
					'system_table' => 'Region',
				),
			),
		),
		
		'workAddress' => array(
			'type' => self::FIELD_TYPE_GROUP,
			'items' => array(
				'streetName' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'visitAddress',
				),
				'streetNumber' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'visitNumber',
				),
				'city' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'visitCity',
				),
				'postalCode' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'visitPostalCode',
				),
				'country' => array(
					'type' => self::FIELD_TYPE_SYSTEM,
					'mapsTo' => 'VisitCountryNode',
					'system_table' => 'Country',
				),
				'region' => array(
					'type' => self::FIELD_TYPE_SYSTEM,
					'mapsTo' => 'VisitProvinceNode',
					'system_table' => 'Region',
				),
			),
		),
		
		'alternativeAddress' => array(
			'type' => self::FIELD_TYPE_GROUP,
			'items' => array(
				'streetName' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'alternativeAddress',
				),
				'streetNumber' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'alternativeNumber',
				),
				'city' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'alternativeCity',
				),
				'postalCode' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'alternativePostalCode',
				),
				'country' => array(
					'type' => self::FIELD_TYPE_SYSTEM,
					'mapsTo' => 'AlternativeCountryNode',
					'system_table' => 'Country',
				),
				'region' => array(
					'type' => self::FIELD_TYPE_SYSTEM,
					'mapsTo' => 'AlternativeProvinceNode',
					'system_table' => 'Region',
				),
			),
		),
		
		'wishes' => array(
			'type' => self::FIELD_TYPE_GROUP,
			'items' => array(
				'employment' => array(
					'type' => self::FIELD_TYPE_SYSTEM,
					'mapsTo' => 'PrefContractTypeNode',
					'system_table' => 'Employment',
				),
				'hoursPerWeek' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'hoursPerWeek',
				), // [todo]
				'desiredFunction1' => array(
					'type' => self::FIELD_TYPE_SYSTEM,
					'mapsTo' => 'PrefBranche1Level1Node',
					'system_table' => 'Business line',
				),
				'desiredFunction2' => array(
					'type' => self::FIELD_TYPE_SYSTEM,
					'mapsTo' => 'PrefBranche2Level1Node',
					'system_table' => 'Business line',
				),
				'desiredFunction3' => array(
					'type' => self::FIELD_TYPE_SYSTEM,
					'mapsTo' => 'PrefBranche3Level1Node',
					'system_table' => 'Business line',
				),
			),
		),
		
		'ambitionAndAvailability' => array(
			'type' => self::FIELD_TYPE_GROUP,
			'items' => array(
				'ambition' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'ambition',
				),
				'availableIn' => array(
					'type' => self::FIELD_TYPE_SYSTEM,
					'mapsTo' => 'AvailableWithinNode',
					'system_table' => 'Notice period',
				),
				'availableFrom' => array(
					'type' => self::FIELD_TYPE_DATE,
					'mapsTo' => 'AvailableFromDate',
				),
			),
		),
		
		'workExperience' => array(
			'type' => self::FIELD_TYPE_GROUP,
			'items' => array(
				'employer' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'employer',
				),
				'periodOf' => array(
					'type' => self::FIELD_TYPE_DATE,
					'mapsTo' => 'periodOf',
				),
				'periodUntil' => array(
					'type' => self::FIELD_TYPE_DATE,
					'mapsTo' => 'periodUntil',
				),
				'functionName' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'functionName',
				),
				'workLocation' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'workLocation',
				),
				'industry' => array(
					'type' => self::FIELD_TYPE_SYSTEM,
					'mapsTo' => 'industry',
					'system_table' => 'Branch0',
				),
				'assignment' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'assignment',
				),
				'otherInfo' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'otherInfo',
				),
			),
		),
		
		'transportation' => array(
			'type' => self::FIELD_TYPE_GROUP,
			'items' => array(
				'homeWork' => array(
					'type' => self::FIELD_TYPE_SYSTEM,
					'mapsTo' => 'homeWork',
					'system_table' => 'Means of transportation',
				),
				'haveCar' => array(
					'type' => self::FIELD_TYPE_CHECK,
					'mapsTo' => 'hasCar',
				),
				'license' => array(
					'type' => self::FIELD_TYPE_SYSTEM,
					'mapsTo' => 'DriverLicenceNode',
					'system_table' => 'Drivers license',
				),
				'issuingPlace' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'issuingPlace',
				),
				'licenceNo' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'licenceNo',
				),
				'numberValidity' => array(
					'type' => self::FIELD_TYPE_DATE,
					'mapsTo' => 'numberValidity',
				),
			),
		),
		
		'education' => array(
			'type' => self::FIELD_TYPE_GROUP,
			'items' => array(
				'education' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'education',
				),
				'institute' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'institute',
				),
				'yearStart' => array(
					'type' => self::FIELD_TYPE_DATE,
					'mapsTo' => 'yearStart',
				),
				'yearEnd' => array(
					'type' => self::FIELD_TYPE_DATE,
					'mapsTo' => 'yearEnd',
				),
				'educationOther' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'educationOther',
				),
			),
		),
		
		'professionalSkills' => array(
			'type' => self::FIELD_TYPE_SYSTEM,
			'mapsTo' => 'professionalSkills',
			'system_table' => 'Skill0',
		),
		
		'languages' => array(
			'type' => self::FIELD_TYPE_SYSTEM,
			'mapsTo' => 'language',
			'system_table' => 'Language',
		),
		
		'motivationAndSource' => array(
			'type' => self::FIELD_TYPE_GROUP,
			'items' => array(
				'motivation' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'motivation',
				), 
				'source' => array(
					'type' => self::FIELD_TYPE_SYSTEM,
					'mapsTo' => 'source',
					'system_table' => 'Source',
				),
				'sourceOther' => array(
					'type' => self::FIELD_TYPE_TEXT,
					'mapsTo' => 'sourceOther',
				),
			),
		),
		
		'rememberMe' => array(
			'type' => self::FIELD_TYPE_CHECK,
			'mapsTo' => 'rememberMe',
		),
	);
	
	static $DEFAULT_SETTINGS = array(
		'is_default' => 'on',
		
		'lastName_visible' => 'on',
		'lastName_required' => 'on',
		
		'firstName_visible' => 'on',
		'firstName_required' => 'on',
		
		/*'password_visible' => 'on',*/
		
		'gender_visible' => 'on',
		
		'birthDate_visible' => 'on',
		'birthDate_required' => 'on',
		
		'cv_visible' => 'on',
		'cv_required' => 'on',
		
		'photo_visible' => 'on',
		
		'email_visible' => 'on',
		'email_required' => 'on',
		
		/* 'homePhone_visible' => 'on', */
		/* 'workPhone_visible' => 'on', */
		'mobileNumber_visible' => 'on',

		'homeAddress_visible' => 'on',
		
		/* 'rememberMe_visible' => 'on', */
	);
	
	
	static $labelKeyForSection  = array(
			'lastName' => 'general.message.field.lastName',
			'firstName' => 'general.message.field.firstName',
			'lastNameAtBirth' => 'general.message.field.lastNameBirth',
			'title' => 'general.message.field.title',
			'suffix' => 'general.message.field.adding',
			'password' => 'general.message.field.password',
			'gender' => 'general.message.field.gender',
			'maritalStatus' => 'general.message.field.maritalStatus',
			'birthDate' => 'general.message.field.birthDate',
			'birthPlace' => 'general.message.field.birthCity',
			'countryOfBirth' => 'general.message.field.birthCountry',
			'nationality' => 'general.message.field.nationality',
			'functionLevel' => 'general.message.field.functionLevel',
			'experience' => 'general.message.field.experience',
			'cv' => 'CxPortal.message.field.cv1',
			'photo' => 'CxPortal.message.field.photo',
			'coverLetter' => 'CxPortal.message.field.coverLetter',
			'email' => 'general.message.field.email',
			'homePhone' => 'general.message.field.homePhone',
			'workPhone' => 'general.message.field.workPhone',
			'mobileNumber' => 'general.message.field.mobileNumber', // unused?
			'mobileNumberBusiness' => 'general.message.field.mobileNumberBusiness',  // unused?
			'contactInstructions' => 'general.message.field.contactInstructions',
			'homeAddress' => 'CxPortal.header.addressPrivate',
			'workAddress' => 'CxPortal.header.addressWork',
			'alternativeAddress' => 'CxPortal.header.addressAlternative',
			'wishes' => 'CxPortal.header.wishes',
			'ambitionAndAvailability' => 'CxPortal.header.ambitionAvailability',
			'workExperience' => 'CxPortal.header.experience',
			'transportation' => 'CxPortal.header.transportation',
			'education' => 'CxPortal.header.educations',
			'professionalSkills' => 'CxPortal.header.professionalSkills',
			'languages' => 'CxPortal.header.languages',
			'motivationAndSource' => 'CxPortal.header.motivationSource',
			'rememberMe' => 'general.message.field.rememberMe',
			'streetName' => 'general.message.field.streetName',
			'streetNumber' => 'general.message.field.streetNo',
			'suffixNumber' => 'general.message.field.adding',
			'city' => 'general.message.field.city',
			'postalCode' => 'general.message.field.zipCode',
			'country' => 'general.message.field.country',
			'region' => 'general.message.field.region',
			'employment' => 'CxPortal.message.field.employment',
			'hoursPerWeek' => 'CxPortal.message.field.weekWorkHours',
			'desiredFunction1' => 'CxPortal.message.field.toLevel1Function1',
			'desiredFunction2' => 'CxPortal.message.field.toLevel1Function2',
			'desiredFunction3' => 'CxPortal.message.field.toLevel1Function3',
			'ambition' => 'CxPortal.message.field.ambition',
			'availableIn' => 'CxPortal.message.field.availableIn',
			'availableFrom' => 'CxPortal.message.field.availableFrom',
			'ambitionAndAvailability' => 'CxPortal.header.ambitionAvailability',
			'employer' => 'CxPortal.message.field.employer',
			'periodOf' => 'CxPortal.message.field.experiencePeriod',
			'periodUntil' => 'CxPortal.message.field.until',
			'functionName' => 'CxPortal.message.field.functionName',
			'workLocation' => 'CxPortal.message.field.workLocation',
			'industry' => 'CxPortal.message.field.industry',
			'assignment' => 'CxPortal.message.field.assignment',
			'otherInfo' => 'CxPortal.message.field.otherExperience',
			'otherInfo' => 'CxPortal.message.field.otherExperience',
			'homeWork' => 'CxPortal.message.field.transportHomeWork',
			'haveCar' => 'CxPortal.message.field.transportHasCar',
			'license' => 'CxPortal.message.field.transportDriveLicence',
			'issuingPlace' => 'CxPortal.message.field.transportPlaceIssueLicence',
			'licenceNo' => 'CxPortal.message.field.transportationLicenceNo',
			'numberValidity' => 'CxPortal.message.field.transportNumeberValidUntil',
			'institute' => 'CxPortal.message.field.institute',
			'yearStart' => 'CxPortal.message.field.yearStart',
			'yearEnd' => 'CxPortal.message.field.toYear',
			'educationOther' => 'CxPortal.message.field.otherCourses',
			'motivation' => 'CxPortal.message.field.motivation',
			'source' => 'CxPortal.message.field.foundThrough',
			'sourceOther' => 'CxPortal.message.field.other',
			'apply' => 'general.button.apply',
                        'hobbies' => 'CxPortal.message.field.hobbies'
		);
	
	static function systemAvailability()
	{
		$availability = array();
		try {
			foreach(self::$FIELDS_TO_DISPLAY as $field_name => $field_settings)
			{
				if(    isset($field_settings['type']) 
					&& $field_settings['type'] == self::FIELD_TYPE_SYSTEM)
				{
					$params = array('type' => $field_settings['system_table']);
					$table_values = CRDataNode::listBy($params);
					if(count($table_values) > 0)
					{
						$availability[$field_name] = TRUE;
					}
					else
					{
						$availability[$field_name] = FALSE;
					}
				}
			}
		}
		catch(Carerix\Api\Rest\Exception $care)
		{
			throw new Exception('Datanodes can not be retrieved');
			Utils_Log::write(Utils_Log::TYPE_DATANODE, false, $care);
		}
		
		return $availability;
	}
	
	static function display($settings, $publication_id = false)
	{
		// $input_data = filter_input_array(INPUT_POST, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL);
		$input_data = map_deep( wp_unslash($_POST), 'sanitize_textarea_field' );
		
		$errors = array();
		if(count($input_data) && wp_verify_nonce($input_data['_carerix_nonce_field'], 'apply'))
		{
			if(isset($input_data['linkedinurl_parse']))
			{
				$data = Environment_Parser::parse_linkedin_url($input_data['linkedinurl']);
				self::$parsed_candidate = CREmployee::fromXml($data);
				
			}
			elseif(isset($input_data['cvupload_parse']))
			{
				// should save the file somewhere
				$file_real_path = $_FILES['cvupload']['tmp_name'];
				$filename = $_FILES['cvupload']['name'];
				
				$uploadDirPath = Carerix::$TMPDIR . DIRECTORY_SEPARATOR . Utils_Log::UPLOADS_DIR_CONTAINER;

				$can_not_upload = false;
				if(! is_dir($uploadDirPath) )
				{
					if( ! mkdir($uploadDirPath, 0777, true))
					{
						$can_not_upload = true;
					}
				}

				if(!$can_not_upload)
				{
					$extra = '_cv_parsed_'.date('Y_m_d_H_i_s');
					$destination_path = $uploadDirPath . DIRECTORY_SEPARATOR . $name . $extra;

					move_uploaded_file($file_real_path, $destination_path);
					$file_real_path = $destination_path;
				}
				
				$data= Environment_Parser::parse_cv_upload($file_real_path, $filename);

				self::$parsed_candidate = CREmployee::fromXml($data);
			}
			else 
			{
				$errors = self::process($input_data, $settings, $publication_id);
			}

		}
		
		
		echo '<div id="apply_form">'; 
		echo '<form method="post" enctype="multipart/form-data">';
                wp_nonce_field('apply', '_carerix_nonce_field');

		$logged_in = \Carerix\Cookie\Cookie::getCookie("logged_in", false);
		if( $logged_in == false )
		{
			$apply_mode = new Utils_Template('fragments/frontend/apply_mode');
			$apply_mode->display();
		}
		
		foreach(self::$FIELDS_TO_DISPLAY as $field_name => $field_settings)
		{
			if(isset($settings[$field_name.'_visible']))
			{
				$required_status = false;
				if(isset($settings[$field_name.'_required']))
				{
					$required_status = true;
				}
				
				self::displayInputField($field_name, @$input_data[$field_name], @$errors[$field_name], false, $required_status);
			}
		}
		
		$interfacer = WP_Interfacer::getInstance();
		$labels = Utils_Labels::get($interfacer->get_default_site_language());
		$apply_label = $labels[self::$labelKeyForSection['apply']];
		echo '<input type="submit" value="'.$apply_label.'" /></form></div>';
	}
	
	static function process($input_data, $settings, $publication_id = false)
	{
		$errors = array();
		
		foreach($input_data as $name => $value)
		{
			if(isset($settings[$name.'_required']))
			{
				if(strlen(trim($value)) == 0)
				{
					$errors[$name] = 'Field is required';
				}
			}
		}

		if(count($errors) == 0)
		{
			$interfacer = WP_Interfacer::getInstance();
			
			if($publication_id === FALSE)
			{
				$properties = $interfacer->current_frontend_entity_get_properties();
				$property_name = '_carerix_job';
				if($interfacer->wpml_is_active())
				{
					$property_name .= '_'.$interfacer->get_default_site_language();
				}

				$publication_id = (int)$properties[$property_name][0];
			}
			
			self::build_employee($input_data, $publication_id);
		}

		return $errors;
	}
	
	static function displayInputField($field_name, $value= '', $error = '', $container = false, $is_required = false, $containername = false)
	{
		if($container === FALSE)
		{
			$container = self::$FIELDS_TO_DISPLAY;
		}
		
		$fragment_path = 'fragments/input/text';
		
		$interfacer = WP_Interfacer::getInstance();
		$labels = Utils_Labels::get($interfacer->get_default_site_language());
		
		$class_chain = 'carerix_input';
		if($is_required)
		{
			$class_chain = ' carerix_required';	
		}
		
		$data = array(
			'label' => $labels[self::$labelKeyForSection[$field_name]],
			'name' => $field_name,
			'value' => $value,
			'error' => $error,
			'class_chain' => $class_chain,
		);

		if($containername)
		{
			$data['name'] = "{$containername}[{$field_name}]";
		}
		
		$field_type = $container[$field_name]['type'];
		
		// $input_data = filter_input_array(INPUT_POST, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL);
		$input_data = map_deep( wp_unslash($_POST), 'sanitize_textarea_field' );

		$logged_in = \Carerix\Cookie\Cookie::getCookie("logged_in", false);

		/**
		 * get the data of the login user only if there is not a POST request
		 */

		if($logged_in && count($input_data) == 0
				|| self::$parsed_candidate)
		{
			try {
				if(self::$parsed_candidate) {
					$candidateEmp = self::$parsed_candidate;
					$candidateUser = $candidateEmp->getToUser();
				} else {
					$EMPLOYEE_ID = \Carerix\Cookie\Cookie::getCookie("employee_id", 0);
					$candidateEmp = CREmployee::find($EMPLOYEE_ID);
					$candidateUser = $candidateEmp->getToUser();
				}
			} catch(Carerix\Api\Rest\Exception $care) {
					Utils_Log::write(Utils_Log::TYPE_FETCH_EMPLOYEE, null, $care);
			}
			
			if($candidateEmp)
			{
				switch($field_type)
				{
					case self::FIELD_TYPE_GROUP:
						break;
					case self::FIELD_TYPE_PASSWORD:
						break;
					case self::FIELD_TYPE_FILE:
						break;
					case self::FIELD_TYPE_SYSTEM:
						$method = 'getTo' . ucfirst($container[$field_name]['mapsTo']);
						$obj = null;
						if(is_callable(array($candidateEmp, $method)))
						{
							$obj = $candidateEmp;
						} elseif(is_callable(array($candidateUser, $method)))
						{
							$obj = $candidateUser;
						}
						if($obj)
						{
							try {
								$_temp = $obj->$method();
								if(is_object($_temp))
								{
									$value = $obj->$method()->getDataNodeID();	
								}
							} catch(Exception $ex)
							{
								// [todo] log only for development environment
							}
						}
						
						break;
					default:
						$method = 'get' . ucfirst($container[$field_name]['mapsTo']);
						
						$obj = null;
						if(is_callable(array($candidateEmp, $method)))
						{
							$obj = $candidateEmp;
						} elseif(is_callable(array($candidateUser, $method)))
						{
							$obj = $candidateUser;
						}
						
						if($obj)
						{
							try {
								$value = $obj->$method();	
							} catch (Exception $ex)
							{
								// [todo] log only for development environment
							}
						}
						break;
				}
			}
			$data['value'] = $value;
		}
		$sectionDisplay = false;
		switch($field_type)
		{
			case self::FIELD_TYPE_PASSWORD:
				$fragment_path = 'fragments/input/password';
				break;
			case self::FIELD_TYPE_SYSTEM:
				$data['options'] = self::get_system_select_options($container[$field_name]['system_table']);
				$fragment_path = 'fragments/input/system_select';
				break;
			case self::FIELD_TYPE_GROUP:
				$items = $container[$field_name]['items'];
				
				$section_data = array('title'=>$labels[self::$labelKeyForSection[$field_name]]);
				$section_start = new Utils_Template('fragments/frontend/section_start');
				$section_start->with($section_data)->display();
				$sectionDisplay = true;
				foreach($items as $item_key => $item_settings)
				{
					//self::displayInputField($item_key, $value, $error, $items);
					$value = @$input_data[$field_name][$item_key];

					self::displayInputField($item_key, $value, $error, $items, false, $field_name);
				}
				break;
			case self::FIELD_TYPE_FILE:
				$fragment_path = 'fragments/input/file';
				break;
			case self::FIELD_TYPE_CHECK:
				$fragment_path = 'fragments/input/check';
				break;
			case self::FIELD_TYPE_DATE:
				$fragment_path = 'fragments/input/date';
				break;
			case self::FIELD_TYPE_TEXT:
			default:
				$fragment_path = 'fragments/input/text';
				break;
		}
		if(!$sectionDisplay)
		{
			$input_text_fragment = new Utils_Template($fragment_path);
			$input_text_fragment->with($data)->display();
		}
		
	}
	
	static function getEmployeeSetter($name, $value, $container = false)
	{
		if($container === FALSE)
		{
			$container = self::$FIELDS_TO_DISPLAY;
		}

		if(@$container[$name]['type'] == self::FIELD_TYPE_GROUP)
		{
			foreach($container[$name]['items'] as $key => $settings)
			{
				$_v = @$value[$key];
				list($method, $processedValue) = self::getEmployeeSetter($key, $_v, $container[$name]['items']);
			}
		}
		else
		{
			if(is_string($value) && strlen($value) == 0) return;
			switch(@$container[$name]['type'])
			{
				case self::FIELD_TYPE_SYSTEM:
					$processedValue = CRDataNode::find($value);
					$method = 'setTo' . ucfirst(@$container[$name]['mapsTo']);
					break;
				default:
					$processedValue = $value;
					$method = 'set' . ucfirst(@$container[$name]['mapsTo']);
					break;
			}
		}
		
		return array($method, $processedValue);
	}
	
	static function build_employee($input_data, $publication_id)
	{
		$employee = new CREmployee();
		$user = new CRUser();
		$userSet = false;

		foreach($input_data as $name => $value)
		{
			if(is_string($value) && strlen(trim($value)) == 0) continue;
			try
			{
				list($method, $processedValue) = self::getEmployeeSetter($name, $value);
				
				if(is_callable(array($employee, $method)))
				{
					$employee->$method($processedValue);
				}
				elseif(is_callable(array($user, $method)))
				{
					$userSet = true;
					$user->$method($processedValue);
				}
			}
			catch (Exception $ex)
			{
				// [todo]
				Utils_Log::write(Utils_Log::TYPE_APPLY_BUILD, $employee, $ex);
			}
		}
		
		if($userSet)
		{
			$employee->setToUser($user);
		}
		
		$attachments = array();
		
		if(isset($_FILES['cv']) && file_exists($_FILES['cv']['tmp_name']))
		{
			// move it to uploads storage folder
			$name = $_FILES['cv']['name'];
			$path = $_FILES['cv']['tmp_name'];
			
			$uploadDirPath = Carerix::$TMPDIR . DIRECTORY_SEPARATOR . Utils_Log::UPLOADS_DIR_CONTAINER;
			
			$can_not_upload = false;
			if(! is_dir($uploadDirPath) )
			{
				if( ! mkdir($uploadDirPath, 0777, true))
				{
					$can_not_upload = true;
				}
			}
			
			if(!$can_not_upload)
			{
				$extra = '_'.date('Y_m_d_H_i_s');
				$destination_path = $uploadDirPath . DIRECTORY_SEPARATOR . $name . $extra;

				move_uploaded_file($path, $destination_path);

				// create attachment
				$attach = new CRAttachment();
				$attach->setFilePath($name);
				$attach->setContent(base64_encode(file_get_contents($destination_path)));

				$attachments[] = $attach;
			}
			
		}
		
		if(!empty($attachments))
		{
			$employee->setAttachments($attachments);
		}
		
		$toBuffer = array();
		$exception = false;

		try {
			$params = array('x-cx-pub' => $publication_id);
			$employee->apply($params);
			//$employee->save();
		} catch (Carerix\Api\Rest\Exception $care)
		{
			$toBuffer['error'] = $care->getMessage();
			$exception = $care;
		} catch (Exception $ex)
		{
			$toBuffer['error'] = $ex->getMessage();
			$exception = $ex;
		}
		
		echo '<textarea>'; 
		echo $employee->getLastRequest();
		echo '</textarea>';
		
		echo '<textarea>'; 
		echo $employee->getLastResponse();
		echo '</textarea>';
		
		if(isset($toBuffer['error'])) 
		{
			Utils_Log::write(Utils_Log::TYPE_APPLY_SEND, $employee, $exception);
			$interface = WP_Interfacer::getInstance();

			$toBuffer['lastname'] = @$input_data['lastName'];
			$toBuffer['data'] = serialize($input_data);
			$toBuffer['time'] = $interface->user_current_time();
			
			$buffersTable = Table_Buffer::get();
			$buffersTable->insert($toBuffer);
		}
		
	}
	
	static function get_system_select_options($table)
	{
		$options = array();
		$params = array('type' => $table);
		$table_values = array();
		$exception = false;
		try {
			$table_values = CRDataNode::listBy($params);
		} catch (Carerix\Api\Rest\Exception $care)
		{
			$exception = $care;
		} catch (Exception $ex) 
		{
			$exception = $ex;
		}
		if($exception)
		{
			Utils_Log::write(Utils_Log::TYPE_DATANODE, false, $exception);
		}
		
		foreach ($table_values as $table_item)
		{
			$option= array(
				'value' => $table_item->dataNodeID,
				'label' => $table_item->value,
			);
			array_push($options, $option);
		}
		return $options;
	}

	/**
	 * Generate Open application page
	 *
	 * @deprecated since 2.22.0
	 */
	static function createDefaultPage()
	{
		return;

		$interfacer = WP_Interfacer::getInstance();
		
		$entities = $interfacer->entities_get_where_property(WP_Constants::DEFAULT_APPLY_PAGE_MARKER, true);
        if(count($entities) > 0) {
            $entity_id = $entities[0]->post_id;
            $interfacer->entity_delete($entity_id);
        }

        $formsTable = new WP_Table(Environment_Installation::TABLE_FORMS);
        $all_forms = $formsTable->getAll();
        $default_not_found = true;
        $found_form_id = null;
        foreach($all_forms as $form)
        {
            $form_settings = unserialize($form['settings']);

            if(isset($form_settings['is_default']))
            {
                $found_form_id = $form['id'];
                $default_not_found = false;
                break;
            }
        }

        if(!$default_not_found)
        {
            $data = array();
            $data['post_title'] = 'Open application';
            $data['post_type'] = 'page';
            $data['post_content'] = "[cx_open_application openFormID='$found_form_id']";
            $data['post_status'] = 'publish';
            $id = $interfacer->entity_insert($data);
            $interfacer->entity_set_property($id, WP_Constants::DEFAULT_APPLY_PAGE_MARKER, true);
        }
		
	}
}