<?php

/**
 * Carerix Wordpress Plugin
 *
 * @category settings
 * @author Bogdan Ivanov
 * @copyright Copyright (c) 2013 Carerix.com (http://www.carerix.com)
 * @link http://www.carerix.com
 * @since Sep 3, 2013 3:06:29 PM
 * @version $Rev$
 */

use Carerix\Api\Rest\Entity\CRDataNode;
use Carerix\Api\Rest\Entity\CREmployee;

/**
 * Subscribe class
 * Used for the job alert
 * @uses templates/frontend/forms/subscribe.php
 */
class Entity_Form_Subscribe extends Entity_Form {
    private $interfacer = null;
    private $publication_id = 0;
	
    private $required_fields = array(
        'email', 'lastname', 'functionLevel0'
    );

    public function __construct($publication_id = 0) {
        $this->interfacer = WP_Interfacer::getInstance();
        $this->publication_id = $publication_id;
	    if (Environment_Settings::getFormGenderUnknown() !== 'on') $this->required_fields[] = "gender";
    }

    public function display($params = array()) {
        $backtrace = debug_backtrace();
        foreach ($backtrace as $bkt) {
            if (isset($bkt['file'])) {
                if (basename($bkt['file']) == 'class-frontend.php') {
                    return '';
                }
            }
        }
        $data = array();
        $errors = array();
        $general_error = '';
        $captchaStatus = Environment_Settings::getJASCaptchaStatus();
//        $input_data = filter_input_array(INPUT_POST, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL);
	    $input_data = map_deep( wp_unslash($_POST), 'sanitize_textarea_field' );
	    $interfacer = WP_Interfacer::getInstance();
	    $labels = Utils_Labels::get($interfacer->get_default_site_language());

        if (count($input_data) && wp_verify_nonce($input_data['_carerix_nonce_field'], 'subscribe')) {
            foreach ($this->required_fields as $name) {
                if (isset($input_data[$name]) && strlen(trim($input_data[$name])) == 0) {
                    $errors[$name] = $labels['CxFinder.message.error.pageContent'];
                }
            }

//            if ($captchaStatus == 'on' && wp_cache_get('captcha_match', CARERIX_CACHE_GROUP) === 0) {
//                $errors['captcha'] = $labels['CxWpPlugin.message.captcha'];
//            }

            if ($captchaStatus == 'on') {
	            $passed = Carerix\Captcha\Captcha::hasUserPassedTheTest();
	            if ($passed == false) $errors['captcha'] = $labels['CxWpPlugin.message.captcha'];
            }

            if (count($errors) == 0) {
                $employee = new CREmployee();
                $employee->setEmailAddress($input_data['email']);
                $employee->setFirstName($input_data['firstname']);
                $employee->setLastName($input_data['lastname']);
                $employee->setLastNamePrefix($input_data['lastNamePrefix']);

                if (!empty($input_data['gender'])) {
                    $gender = CRDataNode::find($input_data['gender']);
                    if ($gender) $employee->setToGenderNode($gender);
                }


                if (!empty($input_data['functionLevel0'])) {
                    $functionLevel0 = CRDataNode::find($input_data['functionLevel0']);
                    if ($functionLevel0) $employee->setToProductTypeNode1($functionLevel0);
                }
                
                if (!empty($input_data['functionLevel1'])) {
                    $functionLevel1 = CRDataNode::find($input_data['functionLevel1']);
                    if ($functionLevel1) $employee->setToLevel1Function1($functionLevel1);
                }

                if (!empty($input_data['functionLevel0_2'])) {
                    $functionLevel0_2 = CRDataNode::find($input_data['functionLevel0_2']);
                    if ($functionLevel0_2) $employee->setToProductTypeNode2($functionLevel0_2);
                }

                if (!empty($input_data['functionLevel1_2'])) {
                    $functionLevel1_2 = CRDataNode::find($input_data['functionLevel1_2']);
                    if ($functionLevel1_2) $employee->setToLevel1Function2($functionLevel1_2);
                }

                if (!empty($input_data['functionLevel0_3'])) {
                    $functionLevel0_3 = CRDataNode::find($input_data['functionLevel0_3']);
                    if ($functionLevel0_3) $employee->setToProductTypeNode2($functionLevel0_3);
                }

                if (!empty($input_data['functionLevel1_3'])) {
                    $functionLevel1_3 = CRDataNode::find($input_data['functionLevel1_3']);
                    if ($functionLevel1_3) $employee->setToLevel1Function2($functionLevel1_3);
                }

                
                if (!empty($input_data['prefCountry1Node'])) {
                    $prefCountry1Node = CRDataNode::find($input_data['prefCountry1Node']);
                    if ($prefCountry1Node) {
                    	$employee->setToPrefCountry1Node($prefCountry1Node);

		                if (!empty($input_data['prefRegion1Node'])) {
			                $prefRegion1Node = CRDataNode::find($input_data['prefRegion1Node']);
			                if ($prefRegion1Node) {
				                $employee->setToPrefRegion1Node($prefRegion1Node);
			                }
		                }

		                if (!empty($input_data['prefRegion2Node'])) {
			                $prefRegion2Node = CRDataNode::find($input_data['prefRegion2Node']);
			                if ($prefRegion2Node) {
				                $employee->setToPrefCountry2Node($prefCountry1Node); // Use the country from 1st pref
				                $employee->setToPrefRegion2Node($prefRegion2Node);
			                }
		                }

		                if (!empty($input_data['prefRegion3Node'])) {
			                $prefRegion3Node = CRDataNode::find($input_data['prefRegion3Node']);
			                if ($prefRegion3Node) {
				                $employee->setToPrefCountry3Node($prefCountry1Node); // Use the country from 1st pref
				                $employee->setToPrefRegion3Node($prefRegion3Node);
			                }
		                }
                    }
                }



	            if (!empty($input_data['_agreement_check']) && $input_data['_agreement_check'] === 'on') {
		            $consent_stage_node = new CRDataNode();
		            $consent_stage_node->setAttributes(array('key' => 'tags', 'value' => 'ConsentGivenTag'));
		            $consent_period_node = new CRDataNode();
		            $consent_period_node->setAttributes(array('key' => 'tags', 'value' => 'DefaultForWebTag'));
		            $employee->setToConsentStageNode($consent_stage_node);
		            $employee->setToConsentPeriodNode($consent_period_node);
		            $employee->setPrivacyApprovalDate(date('Y-m-d 12:00:00', time()));
	            }

	            $frequency = $params['frequency'] ?? ""; // optional shortcode parameter
	            switch ( trim(strtolower($frequency)) ) {
		            case "monthly": // Every first day of the month
			            $frequencyTag = "MonthlyTag";
			            break;
		            case "biweekly": // Every Sunday every other week
			            $frequencyTag = "BiWeeklyTag";
			            break;
		            case "weekly": // Sunday, Sunday
			            $frequencyTag = "WeeklyTag";
			            break;
		            case "twiceweekly": // Sunday, Wednesday, Sunday
			            $frequencyTag = "TwiceWeeklyTag";
			            break;
		            case "tridaily": // Sunday, Wednesday, Saturday, Tuesday, Friday, Monday, Thursday, Sunday...
			            $frequencyTag = "TriDailyTag";
			            break;
		            case "bidaily": // Sunday, Tuesday, Thursday, Saturday, Monday, Wednesday, Friday, Sunday...
			            $frequencyTag = "BiDailyTag";
			            break;
		            case "daily": // Every day
		            default:
			            $frequencyTag = "DailyTag";
	            }

	            $transientData = array('group' => 'JobAlertTag', 'frequency' => $frequencyTag);
                if (isset($params['groupid'])) {
                    $transientData['nid'] = $params['groupid'];
                }

                $employee->setTransient($transientData);

                $exception = false;
                try {
                    $employee->subscribe();
                    $data['succesful'] = true;

	                $data['confirmation_message'] = isset($params['confirmation_message']) ? $params['confirmation_message'] : ""; // optional shortcode parameter
	                $data['confirmation_url'] = isset($params['confirmation_url']) ? $params['confirmation_url'] : ""; // optional shortcode parameter

                } catch (Carerix\Api\Rest\Exception $care) {
                    $general_error = $care->getMessage();
                    $exception = $care;
                } catch (Exception $ex) {
                    $general_error = $ex->getMessage();
                    $exception = $ex;
                }
                if ($exception) {
                    Utils_Log::write(Utils_Log::TYPE_JAS, $employee, $exception);
                }
            }
        }

        $data['general_error'] = $general_error;
        $data['errors'] = $errors;

        $data['genders'] = $this->getGenders();
        $data['functionLevels'] = $this->getFunctionLevels();
        $data['countryRegions'] = $this->getCountryRegionNodes();
        $data['agreement'] = $this->_agree_html();

        $data['captcha'] = $captchaStatus;

        $template = new Utils_Template('frontend/forms/subscribe');
        $output = $template->with($data)->display( true );
        return $output;
    }

	private function getGenders()
	{
		$processedGenders = array();

        try {
			$genders = CRDataNode::listBy(array('type'=>'Salutation',
                'show'=>array('value', 'dataNodeID', 'isHidden', 'parentNodes.value'), 'qualifier'=> " isHidden = 0 "));
			foreach($genders as $gender) {
				$blFoundUnknownGender = false;
				foreach($gender->parentNodes as $pn) {
					if($pn->value == 'UnknownTag' ) $blFoundUnknownGender = true;
				}

                if (Environment_Settings::getFormGenderUnknown() !== 'on' && $blFoundUnknownGender) continue;

                if ($blFoundUnknownGender) {
                	// Prepend to list
	                array_unshift($processedGenders, array('id'=>$gender->dataNodeID, 'value'=>''));
                } else {
                	// Append to list
	                $processedGenders[] = array('id'=>$gender->dataNodeID, 'value'=>$gender->value);
                }
            }
		} catch (Exception $ex)
		{
            Utils_Log::write(Utils_Log::TYPE_DATANODE, $genders, $ex);
        }
        return $processedGenders;
    }

	private function getFunctionLevels()
	{
        $cache_session_key = md5('function_level_cache');
        if(FALSE !== $cached_data = Utils_Cache::get($cache_session_key)) return $cached_data;

        $general_exception = false;
        try {
	         $fieldsLevel0 = CRDataNode::listBy(array('type'=>'Business line',
	                                                                    'qualifier'=> ' isHidden = 0'));

            $fieldsHierarchy = array();


			foreach($fieldsLevel0 as $field)
			{
				if ($field->isHidden) continue; // Somehow the qualifier 'isHidden' isn't respected in the API request. So we filter it manually

				$level = array('id'=>$field->dataNodeID, 'value'=>$field->value, 'children'=>array());
				$fieldsLevel1 = CRDataNode::listBy(array('type'=>'Function0', 'parent'=>$field->dataNodeID));

				if($fieldsLevel1 != null) {
					foreach($fieldsLevel1 as $fieldLevel1)
					{
						$level2 = array('id'=>$fieldLevel1->dataNodeID, 'value'=>$fieldLevel1->value, 'children'=>array());

						$fieldsLevel2 = CRDataNode::listBy(array('type'=>'Function1', 'parent'=>$fieldLevel1->dataNodeID));

						if($fieldsLevel2 !== null) {
							foreach($fieldsLevel2 as $fieldLevel2)
							{
								$level3 = array('id'=>$fieldLevel2->dataNodeID, 'value'=>$fieldLevel2->value);
                                array_push($level2['children'], $level3);
                            }
                        }
                        array_push($level['children'], $level2);
                    }
                }

                array_push($fieldsHierarchy, $level);
            }
		}
		catch (Carerix\Api\Rest\Exception $care)
		{
            $general_exception = $care;
		} catch (Exception $ex)
		{
            $general_exception = $ex;
        }

        Utils_Cache::set($cache_session_key, $fieldsHierarchy);
		if($general_exception)
		{
            Utils_Log::write(Utils_Log::TYPE_DATANODE, false, $general_exception);
            return $fieldsHierarchy;
		} else 
		{
            Utils_Cache::set($cache_session_key, $fieldsHierarchy);
            return $fieldsHierarchy;
        }
    }

    /**
     * Returns list of Countries and Regions
     * @return array
     */
    private function getCountryRegionNodes() {
        $cache_session_key = md5('country_region_cache');
        if (FALSE !== $cached_data = Utils_Cache::get($cache_session_key)) {
            return $cached_data;
        }

        $general_exception = false;
        $result = array();
        try {
            $countries = CRDataNode::listBy(array(
                'type' => 'Country',
                'show' => array('value', 'childrenNodes.value', 'isHidden'),
                'qualifier' => "isHidden = 0 and isActive = 1"
            ));

            foreach ($countries as $country) {
            	if ($country->isHidden) continue; // Somehow the qualifier 'isHidden' isn't respected in the API request. So we filter it manually
                $childrens = array();
                if (isset($country->childrenNodes) && count($country->childrenNodes)) {
                    foreach ($country->childrenNodes as $child) {
                        $childrens[] = array('id' => $child->dataNodeID, 'value' => $child->value);
                    }
                }
                $result[] = array('id' => $country->dataNodeID, 'value' => $country->value, 'children' => $childrens);
            }
        } catch (Carerix\Api\Rest\Exception $care) {
            $general_exception = $care;
        } catch (Exception $ex) {
            $general_exception = $ex;
        }

        Utils_Cache::set($cache_session_key, $result);
        if ($general_exception) {
            Utils_Log::write(Utils_Log::TYPE_DATANODE, false, $general_exception);
            return $result;
        } else {
            Utils_Cache::set($cache_session_key, $result);
            return $result;
        }
    }

}
