<?php

use Carerix\Api\Rest\Entity\CRPublication;

/**
 * Carerix Wordpress Plugin
 *
 * @category settings
 * @author Bogdan Ivanov
 * @copyright Copyright (c) 2013 Carerix.com (http://www.carerix.com)
 * @link http://www.carerix.com
 * @since Sep 3, 2013 4:27:07 PM
 * @version $Rev$
 */

class Frontend_Shortcode_Job extends Frontend_Shortcode {
    private ?WP_Interfacer $interfacer = null;
	private array $labels;

	public function __construct() {
        $this->interfacer = WP_Interfacer::getInstance();
    }

	public function register_shortcodes( $dataDictionary = array(), $categoryLanguage = false ): Frontend_Shortcode {
		// new style (fx = attached function, section = defined in getSectionLabelKey() and have to match post meta field key )
		$aspartofvacancypublicationtext = "This is part of the vacancy publication text.";
		$optional_attributes = '<code>container="<b>div</b>|span"</code> wrap output in a div or span tag<br><code style="text-decoration: line-through;">label="<b>yes</b>|no"</code> (deprecated) show heading tag above the content<br><code>heading="h1..<b>h4</b>..h6|div|span|no"</code> use particular heading tag or hide it<br><code>content="<b>yes</b>|no"</code> display the paragraph content<br><code>class=""</code> use specific css class for the heading<br><code>raw="yes|<b>no</b>"</code> set to yes to render only the content<br><code style="text-decoration: line-through;">quotes="<b>double</b>|single"</code> (deprecated) how to embed the heading tag attributes';
		$optional_attributes_raw = '<code>container="div|span"</code> wrap output in a div or span tag<br><code style="text-decoration: line-through;">label="yes|no"</code> (deprecated) show heading tag above the content<br><code>heading="h1..h6|div|span|<b>no</b>"</code> use particular heading tag or hide it<br><code>content="<b>yes</b>|no"</code> display the paragraph content<br><code>class=""</code> use specific css class for the heading<br><code>raw="<b>yes</b>|no"</code> set to yes to render only the content<br><code style="text-decoration: line-through;">quotes="<b>double</b>|single"</code> (deprecated) how to embed the heading tag attributes';
		$xml_sitemap_link = get_bloginfo('url') . '/?carerix_sitemap';

		$this->shortcodeDictionary["cx_intro_information"] =                array( "fx" => "vacancy_information",
		                                                                           "section" => "introInformation",
		                                                                           "attributes" => $optional_attributes,
		                                                                           "descr" => "Displays the vacancy introduction information.<br>{$aspartofvacancypublicationtext}",
		                                                                           "example" =>'[cx_intro_information heading="no"]' );
		$this->shortcodeDictionary["cx_company_information"] =              array( "fx" => "vacancy_information",
		                                                                           "section" => "companyInformation",
		                                                                           "attributes" => $optional_attributes,
																				   "descr" => "Displays the description of the organization and/or department where the vacancy is located.<br>{$aspartofvacancypublicationtext}",
																				   "example" => '[cx_company_information heading="h2"]');
		$this->shortcodeDictionary["cx_vacancy_information"] =              array( "fx" => "vacancy_information",
		                                                                           "section" => "vacancyInformation",
		                                                                           "attributes" => $optional_attributes,
		                                                                           "descr" => "Displays the description of the position.<br>{$aspartofvacancypublicationtext}",
		                                                                           "example" => "[cx_vacancy_information]" );
		$this->shortcodeDictionary["cx_requirements_information"] =         array( "fx" => "vacancy_information",
		                                                                           "section" => "requirementsInformation",
		                                                                           "attributes" => $optional_attributes,
		                                                                           "descr" => "Displays the description of the requirements for the position.<br>{$aspartofvacancypublicationtext}",
		                                                                           "example" => "[cx_requirements_information]" );
		$this->shortcodeDictionary["cx_offer_information"] =                array( "fx" => "vacancy_information",
		                                                                           "section" => "offerInformation",
		                                                                           "attributes" => $optional_attributes,
		                                                                           "descr" => "Displays what the client or intermediary offers the applicant (e.g. salary, bonus, training opportunities).<br>{$aspartofvacancypublicationtext}",
		                                                                           "example" => "[cx_offer_information]" );
		$this->shortcodeDictionary["cx_end_date"] =                         array( "fx" => "vacancy_information",
		                                                                           "section" => "endDate",
		                                                                           "_type" => "date",
		                                                                           "attributes" => $optional_attributes,
		                                                                           "descr" => "Displays the publication end date. Please consider using <code>[cx_apply_until]</code> if you want to show the date of the final day applicants can apply.",
																					"example" => '[cx_end_date label="no" raw="no" container="span"] nicely date formatted<br><br>[cx_end_date raw="yes"] raw ODBC date');
		$this->shortcodeDictionary["cx_function_contact_information"] =     array( "fx" => "vacancy_information",
		                                                                           "section" => "functionContactInformation",
		                                                                           "attributes" => $optional_attributes,
		                                                                           "descr" => "Displays the contact details for the applicant to obtain more information about the position and the organization.<br>{$aspartofvacancypublicationtext}",
		                                                                           "example" => "[cx_function_contact_information]");
		$this->shortcodeDictionary["cx_application_contact_information"] =  array( "fx" => "vacancy_information",
		                                                                           "section" => "applicationContactInformation",
		                                                                           "attributes" => $optional_attributes,
		                                                                           "example" => '[cx_application_contact_information heading="h2"]',
		                                                                           "descr" => "Displays the person at the client who serves as the first point of contact.<br>{$aspartofvacancypublicationtext}" );
		$this->shortcodeDictionary["cx_organization"] =                       array( "fx" => "vacancy_information",
		                                                                       "section" => "company",
		                                                                      "attributes" => $optional_attributes_raw,
		                                                                      "heading" => "no",
		                                                                      "raw" => "yes",
		                                                                       "descr" => "Displays the organization/company name of the vacancy.",
		                                                                       "example" => "[cx_organization]");
		$this->shortcodeDictionary["cx_field"] =                       array( "fx" => "vacancy_information",
		                                                                       "section" => "businessLine",
		                                                                      "attributes" => $optional_attributes_raw,
		                                                                      "heading" => "no",
		                                                                      "raw" => "yes",
		                                                                       "descr" => "Displays the field (area of expertise) of the vacancy.",
		                                                                       "example" => "[cx_field]");
		$this->shortcodeDictionary["cx_job_title"] =                       array( "fx" => "vacancy_information",
		                                                                       "section" => "rawTitle",
		                                                                      "attributes" => $optional_attributes_raw,
																				"heading" => "no",
																				"raw" => "yes",
		                                                                       "descr" => "Displays the job title of the vacancy.",
		                                                                       "example" => "[cx_job_title]");
		$this->shortcodeDictionary["cx_country"] =                          array( "fx" => "vacancy_information",
		                                                                       "section" => "country",
		                                                                      "_override_data_dictionary_key" => "country1,country2,country3",
																			  "_source" => "taxonomy", // instruct to grab the info from post taxonomy
		                                                                      "attributes" => '<code>separator="<b>,&nbsp;</b>"</code> separate each country by a comma or another character.<br>' . $optional_attributes_raw,
																				"heading" => "no",
																				"raw" => "yes",
		                                                                       "descr" => "Displays the country or countries of the vacancy.",
		                                                                       "example" => "[cx_country]");
		$this->shortcodeDictionary["cx_region"] =                          array( "fx" => "vacancy_information",
		                                                                       "section" => "region",
		                                                                          "_override_data_dictionary_key" => "region1,region2,region3",
																			  "_source" => "taxonomy", // instruct to grab the info from post taxonomy
		                                                                      "attributes" => '<code>separator="<b>,&nbsp;</b>"</code> separate each region by a comma or another character.<br>' . $optional_attributes_raw,
																				"heading" => "no",
																				"raw" => "yes",
		                                                                       "descr" => "Displays the region or regions of the vacancy.",
		                                                                       "example" => "[cx_region]");
		$this->shortcodeDictionary["cx_work_location"] =                       array( "fx" => "vacancy_information",
		                                                                       "section" => "workLocation",
		                                                                      "attributes" => $optional_attributes_raw,
																				"heading" => "no",
																				"raw" => "yes",
		                                                                       "descr" => "Displays the work location of the vacancy.",
		                                                                       "example" => "[cx_work_location]");
		$this->shortcodeDictionary["cx_work_postal_code"] =                    array( "fx" => "vacancy_information",
		                                                                       "section" => "postalCode",
		                                                                      "attributes" => $optional_attributes_raw,
																				"heading" => "no",
																				"raw" => "yes",
		                                                                       "descr" => "Displays the postal code of the work location. It's empty if the job order is set to anonymous in the Carerix system.",
		                                                                       "example" => "[cx_work_postal_code]");
		$this->shortcodeDictionary["cx_workplace_type"] =                       array( "fx" => "workplaceType",
		                                                                       "section" => "workplaceTypeClassification",
		                                                                       "descr" => "Displays the workplace type, e.g. <i>Remote</i>, <i>On site</i> or <i>Hybrid</i>",
		                                                                       "example" => "[cx_workplace_type]");
		$this->shortcodeDictionary["cx_office_name"] =                       array( "fx" => "vacancy_information",
		                                                                       "section" => "agency",
		                                                                      "attributes" => $optional_attributes_raw,
	                                                                            "heading" => "no",
	                                                                            "raw" => "yes",
		                                                                       "descr" => "Displays the office name which is connected to the vacancy.",
		                                                                       "example" => "[cx_office_name]");
		$this->shortcodeDictionary["cx_hours_per_week"] =              array( "fx" => "vacancy_information",
		                                                                       "section" => "hoursPerWeek",
		                                                                      "attributes" => $optional_attributes_raw,
		                                                                      "heading" => "no",
		                                                                      "raw" => "yes",
		                                                                       "descr" => 'Displays the working hours per week.',
		                                                                       "example" => "[cx_hours_per_week]");
		$this->shortcodeDictionary["cx_salary"] =                       array( "fx" => "salary",
																			   "attributes" => '<code>show="<b>csu</b>"</code> include these elements, c=currency, s=salary (range), u=(period) unit<br><code>raw="yes|<b>no</b>"</code> render as (un)formatted text',
		                                                                       "descr" => 'Displays the salary range. In the Carerix system the "publish salary" checkbox must be checked.',
		                                                                       "example" => '[cx_salary] e.g. € 2000-2500 per month<br><br>[cx_salary show="s"] e.g. 2000-2500<br><br>[cx_salary show="c" raw="yes"] e.g. EUR');
		$this->shortcodeDictionary["cx_employment"] =                  array( "fx" => "vacancy_information",
		                                                                       "section" => "contractType",
		                                                                      "attributes" => $optional_attributes_raw,
		                                                                      "heading" => "no",
		                                                                      "raw" => "yes",
		                                                                       "descr" => 'Displays the contract type of employment, e.g. Temporarily contract.',
		                                                                       "example" => "[cx_employment]");
		$this->shortcodeDictionary["cx_professional_level"] =                  array( "fx" => "vacancy_information",
		                                                                       "section" => "toWorkLevelNode",
		                                                                      "attributes" => $optional_attributes_raw,
		                                                                      "heading" => "no",
		                                                                      "raw" => "yes",
		                                                                       "descr" => 'Displays the required professional level.',
		                                                                       "example" => "[cx_professional_level]");
		$this->shortcodeDictionary["cx_education"] =                        array( "fx" => "vacancy_information",
		                                                                           "section" => WP_Constants::TAXONOMY_EDUCATION,
																				   "_override_data_dictionary_key" => "educations",
		                                                                           "_source" => "taxonomy", // instruct to grab the info from post taxonomy
		                                                                           "attributes" => '<code>separator="<b>,&nbsp;</b>"</code> separate each education by a comma or another character.<br>' . $optional_attributes_raw,
		                                                                           "heading" => "no",
		                                                                           "raw" => "yes",
		                                                                           "descr" => 'Displays the educational requirements of the vacancy.',
		                                                                           "example" => "[cx_education]");
		$this->shortcodeDictionary["cx_procedure"] =                  array( "fx" => "vacancy_information",
		                                                                       "section" => "procedure",
		                                                                      "attributes" => $optional_attributes_raw,
		                                                                      "heading" => "no",
		                                                                      "raw" => "yes",
		                                                                       "descr" => 'Displays the prodecure of the vacancy, e.g. Secondment or Recruitment.',
		                                                                       "example" => "[cx_procedure]");
		$this->shortcodeDictionary["cx_sector"] =                       array( "fx" => "vacancy_information",
		                                                                       "section" => "branche0",
		                                                                       "attributes" => $optional_attributes_raw,
		                                                                       "heading" => "no",
		                                                                       "raw" => "yes",
		                                                                       "descr" => 'Displays the sector of the vacancy, e.g. Health care.',
		                                                                       "example" => "[cx_sector]");
		$this->shortcodeDictionary["cx_industry"] =                       array( "fx" => "vacancy_information",
		                                                                       "section" => "branche1",
		                                                                         "attributes" => $optional_attributes_raw,
		                                                                         "heading" => "no",
		                                                                         "raw" => "yes",
		                                                                       "descr" => 'Displays the industry of the vacancy, e.g. <i>Accountancy</i> (as part of the sector <i>Business services</i>).',
		                                                                       "example" => "[cx_industry]");

		$this->shortcodeDictionary["cx_function_group"] =                   array( "fx" => "vacancy_information",
		                                                                           "section" => "function0",
		                                                                           "attributes" => $optional_attributes_raw,
		                                                                           "heading" => "no",
		                                                                           "raw" => "yes",
		                                                                           "descr" => 'Function group of the vacancy, for example "ICT professionals."',
		                                                                           "example" => '[cx_function_group]' );
		$this->shortcodeDictionary["cx_function_position"] =                array( "fx" => "vacancy_information",
		                                                                           "section" => "function1",
		                                                                           "attributes" => $optional_attributes_raw,
		                                                                           "heading" => "no",
		                                                                           "raw" => "yes",
		                                                                           "descr" => 'The position within the function group to which the vacancy belongs, for example "Network administrator" from the group "ICT professionals".',
		                                                                           "example" => '[cx_function_position]' );
		$this->shortcodeDictionary["cx_job_group"] =                        array( "fx" => "vacancy_information",
		                                                                          "section" => WP_Constants::TAXONOMY_JOBGROUP,
		                                                                           "_override_data_dictionary_key" => "groups",
		                                                                          "_source" => "taxonomy", // instruct to grab the info from post taxonomy
		                                                                          "attributes" => '<code>separator="<b>,&nbsp;</b>"</code> separate each group by a comma or another character.<br>' . $optional_attributes_raw,
		                                                                          "heading" => "no",
		                                                                          "raw" => "yes",
		                                                                          "descr" => 'Displays the group(s) the vacancy is appointed to. Don\'t confuse this one with [cx_function_group]. Groups are free to be defined in the Carerix system. A useful group for website purposes is <i>top vacancies</i> to use for a vacancy carousel.',
		                                                                          "example" => "[cx_job_group]");
		$this->shortcodeDictionary["cx_extra"] =                            array( "fx" => "vacancy_additional_field",
		                                                                           "attributes" => '<code>* field="yourcarerixfieldcode"</code> the configured code (ID) of the field to identify it in the Carerix system<br><code>separator="<b>,&nbsp;</b>"</code> separate each item by a comma or another character. Useful for outputing multiple entries like checkboxes<br><code>type="<b>string</b>|date"</code> formats a date field as configured in the general WordPress settings',
		                                                                           "descr" => "Displays an extra/additional field that is activated in joborders or publications in the Carerix system.<br><br>Notes:<br>1) This is an advanced feature. Prepared configuration in the Carerix system is required.<br>2) In the settings of this plugin extra/additional field support must be enabled.",
		                                                                           "example" => '[cx_extra field="coffee_preference"]<br><br>[cx_extra field="coffee_purchase_date" type="date"]');
		$this->shortcodeDictionary["cx_google_for_jobs"] =                  array( "fx" => "vacancy_gfj",
		                                                                           "descr" => "Forcefully generates JobPosting structured data as application/ld+json script. Only use this shortcode if you are <i>completely</i> overriding WordPress single-post templates. This may be the case when using page builders like Elementor or Divi to display the vacancy information pages.",
		                                                                           "example" => "[cx_google_for_jobs]" );


		// old style
		$this->shortcodeDictionary["cx_apply_until"] =                  array( "fx" => "apply_until",
		                                                                       "descr" => "Displays the final apply date in the date format configured in the general WordPress settings.",
		                                                                       "example" => '[cx_apply_until]' );

		$this->shortcodeDictionary["cx_rss_for_category"] =             array( "fx" => "rss_for_category",
		                                                                       "deprecated" => "3.1.0",
		                                                                       "descr" => "Displays a link to the RSS feed of the current posts-category.<br>Instead, use {$xml_sitemap_link} as a vacancy sitemap.",
		                                                                       "example" => "[cx_rss_for_category]" );
		$this->shortcodeDictionary["cx_rss_for_all"] =                  array( "fx" => "rss_for_all",
		                                                                       "deprecated" => "3.1.0",
		                                                                       "descr" => "Displays a link to the RSS feed of all the content.<br>Instead, use {$xml_sitemap_link} as a vacancy sitemap.",
		                                                                       "example" => "[cx_rss_for_all]" );
		$this->shortcodeDictionary["cx_print_link"] =                   array( "fx" => "print_link",
		                                                                       "deprecated" => "3.1.0",
		                                                                       "descr" => "Displays a link to physically print the current page.",
		                                                                       "example" => "[cx_print_link]" );
		// $this->shortcodeDictionary["cx_function_group"] = array( "fx" => "function_group_legacy" ); // Legacy: includes function0 + function1 and uses an API call
		$this->shortcodeDictionary["cx_vacancy_number"] =               array( "fx" => "vacancy_number",
		                                                                       "deprecated" => "3.1.0",
		                                                                       "descr" => 'Displays the vacancy ID. Please use <code>[vacancy_code]</code> instead.',
		                                                                       "example" => "[vacancy_number]" );
		$this->shortcodeDictionary["cx_vacancy_code"] =                 array( "fx" => "vacancy_code",
		                                                                       "attributes" => '<code>type="<b>code</b>|id"</code> use the reference code or the ID',
		                                                                       "descr" => 'Displays either the vacancy reference code or the vacancy ID. The reference code can be manually set in the Carerix system, but defaults to the (autogenerated) vacancy ID aka "vacancy number".',
		                                                                       "example" => "[cx_vacancy_code]" );
		$this->shortcodeDictionary["cx_medium"] =                 array( "fx" => "vacancy_medium",
		                                                                       "attributes" => '<code>type="<b>code</b>"</code> uses the mediumcode, other options reserved for future changes',
																			   "type" => "code",
		                                                                       "descr" => 'Displays medium-code of the Carerix publication of the job post.',
		                                                                       "example" => "[cx_medium]" );
		$this->shortcodeDictionary["cx_publication_id"] =                 array( "fx" => "publication_id",
		                                                                       "descr" => 'Displays the Carerix publicaton ID of the current job post.',
		                                                                       "example" => "[cx_publication_id]" );
		$this->shortcodeDictionary["cx_social_icons"] =                 array( "fx" => "social_icons",
		                                                                       "deprecated" => "3.1.0",
		                                                                       "descr" => "Displays share this page buttons (Facebook, X and LinkedIn) generated by sharethis.com. It's better to use social share-options from your WordPress theme.",
		                                                                       "example" => "[cx_social_icons]" );
		$this->shortcodeDictionary["cx_apply_button"] =                 array( "fx" => "apply_button",
																				"descr" => "Displays the apply button which links to the application form on a separate URL. Typically, this shortcode is used under the vacancy information.<br>To display the applicaton form and avoid a separate application form page, use <code>[cx_apply_form]</code> instead.",
																				"example" => "[cx_apply_button]");

		$this->shortcodeDictionary["cx_apply_form"] =                   array( // Note: fx() is intentially left out here because the shortocde functionality it's defined elsewhere. So this is for documentation purposes only
								                                               "descr" => "Displays the application form for the current vacancy post. Typically use this to display both the vacancy information and application form at the same page.",
								                                               "example" => "[cx_apply_form]");
		$this->shortcodeDictionary["cx_open_application"] =             array( // Note: fx() is intentially left out here because the shortocde functionality it's defined elsewhere. So this is for documentation purposes only
																				"attributes" => '<code>* openformid="x"</code> replace x by the application form ID<br><code>openpubid="y"</code> optionally force to use a different Carerix publication-ID',
								                                               "descr" => 'Displays the application form for an open applicaton. In <i>Dashboard->Carerix->Application Forms</i> you can find the form ID.<br>For this functionality to work, in the Carerix system, a published job order marked as an "open application" must be present.',
								                                               "example" => '[cx_open_application openFormID="1"]');
		$this->shortcodeDictionary["cx_job_alert_subscription"] =       array( // Note: fx() is intentially left out here because the shortocde functionality it's defined elsewhere. So this is for documentation purposes only
																				"attributes" => '<code>confirmation_message="abc"</code> replace abc by a suiteable UI message which appears after a succesful signup<br><code>confirmation_url="abc"</code> optionally redirect to a specific (relatieve) URL after a sucessful signup<br><code>frequency="<b>daily</b>|bidaily|tridaily|twiceweekly|weekly|biweekly|monthly"</code> optionally specify a different default time schedule for the subscriber',
								                                               "descr" => 'Displays the job alert signup form.<br>After filling in, the subscriber will first receive an email (sent from the Carerix system) with a mandatory confirmation link before the subscription becomes active.<br><br>Meaning of the frequency<br>
daily: Every day<br>bidaily: Sunday, Tuesday, Thursday, Saturday, Monday, Wednesday, Friday, Sunday...<br>tridaily: Sunday, Wednesday, Saturday, Tuesday, Friday, Monday, Thursday, Sunday...<br>twiceweekly: Sunday, Wednesday, Sunday...<br>weekly: Sunday, Sunday...<br>biweekly: Every Sunday every other week<br>monthly: Every first day of the month',
								                                               "example" => '[cx_job_alert_subscription]<br><br>[cx_job_alert_subscription confirmation_url="/thank_you/" frequency="weekly"]');
		$this->shortcodeDictionary["cx_campaign"] =                     array( // Note: fx() is intentially left out here because the shortocde functionality it's defined elsewhere. So this is for documentation purposes only
																				"deprecated" => "3.1.0",
																				"attributes" => '<code>* sendto="email@addre.ss"</code> notification emailadress for campaign manager<br><code>campaignid="x"</code> set to only display this campaign form<br><code>codeinname="abc"</code> only display matching campaign-statuses',
								                                               "descr" => 'Displays a <i>Carerix campaign</i> signup form. Only active campaigns (end-date not expired) are listed.<br><br>Note: the intended functionality may no longer work due to functional changes in the Carerix system.',
								                                               "example" => '[cx_campaign sendto="recruiter@yourcompany.com"] signup form containing all available campaigns to select<br><br>[cx_campaign sendto="recruiter@yourcompany.com" campaignid="1234"] signup form for a particular campaign');
		$this->shortcodeDictionary["cx_login_form"] =                   array( // Note: fx() is intentially left out here because the shortocde functionality it's defined elsewhere. So this is for documentation purposes only
																				"deprecated" => "3.1.0",
								                                               "descr" => 'Displays a login form for Carerix candidates.<br><br>Note: the intended functionality may no longer work due to functional changes in the Carerix system.',
								                                               "example" => '[cx_login_form]');
		$this->shortcodeDictionary["cx_newsletter_subscription"] =      array( // Note: fx() is intentially left out here because the shortocde functionality it's defined elsewhere. So this is for documentation purposes only
																				"deprecated" => "3.1.0",
																				"attributes" => '<code>* newsletterID="x"</code> set x to the corresponding newsletter ID listed in the Carerix system.',
								                                               "descr" => 'Displays a newsletter signup form for existing newsletters in the Carerix system. <br><br>Note: the intended functionality may no longer work due to functional changes in the Carerix system.',
								                                               "example" => '[cx_newsletter_subscription newsletterID="1"]');

		// Utils_Log::info("\n*register_shortcodes:*\n" . print_r($dataDictionary, true));

		parent::register_shortcodes( $dataDictionary );

		$this->labels = $this->getLabels( $categoryLanguage );

		return $this;
	}


	/**
	 * @param $atts
	 * @param $content
	 * @param array $dataDictionary The value is stored in the dataDictionary if used in a template of "CX Plugin->Sources". If shortcode is used on front-end the dataDictionary array is empty
	 *
	 * @return string
	 */
	public function vacancy_information( $atts, $content, $dataDictionary ) {

//		Utils_Log::info("\n 1: " . print_r($atts, true) . "\n");

		$atts = shortcode_atts( array(
			'shortcode_name' => '',
			'section_key' => '', // allowed to be changed in shortcode usage
			'_override_data_dictionary_key' => '', // used in case when section_key isn't available as key (or comma-separated keys) in data dictionary
			'_type' => 'string',
			'_source' => 'meta', // grab information from post 'meta' or post 'taxonomy'
			'label' => 'yes', // render the heading
			'heading' => 'h4', // can be h1..h6|span|div|no
			'container' => 'div',
			'content' => 'yes', // render the paragraph content
			'separator' => ', ', // use this seperator to list multiple values. E.g. for a vacancy in multiple countries. Shortcode: [cx_country]
			'class' => '', // prepend custom css classes
			'quotes' => 'double', // legacy renderer uses single quotes for HTML attributes
			'raw' => 'no', // raw mode will leave out added html-tags
		), $atts );

		$section_key = $atts['section_key']; // E.g. "introInformation";

//		Utils_Log::info("\n" . print_r($atts, true) . "\n");
//		Utils_Log::info("\n" . print_r($dataDictionary, true) . "\n");

		if ( count($dataDictionary) && strlen($atts['_override_data_dictionary_key']) ) { // Preferred method
			$dataDictionaryKeys = explode(",", $atts['_override_data_dictionary_key'] );
			$collectedValues = array();
			foreach ($dataDictionaryKeys as $dataDictionaryKey) {
				$values = $dataDictionary[ $dataDictionaryKey ] ?? array();
				if ( is_string($values) ) $values = array($values);
				foreach ($values as $value) {
					$collectedValues[] = $value;
				}
			}
			$value = count($collectedValues) ? implode($atts['separator'], $collectedValues) : "";
		} elseif ( count($dataDictionary) && strlen($section_key) ) {
			$value = $dataDictionary[$section_key] ?? "";
		} elseif ($atts['_source'] === "meta") {
			$value = $dataDictionary[$section_key] ?? get_post_meta(get_the_ID(), $section_key, true) ?? "";
		} elseif ($atts['_source'] === "taxonomy") {
			$terms = get_the_terms(get_the_ID(), $section_key);
			$value = implode($atts['separator'], $terms===FALSE ? array() : array_column($terms, "name"));
		} else {
			$value = '<span style="color: red">invalid _source provided!</span>';
		}

		if ( strlen(trim($value)) == 0 ) return "";


		$show_label = $this->yesNoToBool(trim($atts['label']));
		$show_content = $this->yesNoToBool(trim($atts['content']));
		$heading = strtolower(trim($atts['heading'])); // (int) preg_replace('/[^0-9]+/', '', trim($atts['heading']));
		$attr_quote = ( trim(strtolower($atts['quotes'])) == "single" ) ? "'" : '"'; // support single quote for legacy purposes
		$show_raw = $this->yesNoToBool(trim($atts['raw']), false);

		// $contentSections = $this->getContentSections();
		$label_key = $this->getSectionLabelKey($section_key);
		$section_name = $this->labels[$label_key] ?? "";
		$css_class = trim($atts['class'] . " cx2_" . @strtolower(@end(@explode(".", $label_key))));

		if (!$show_raw) {
			switch ($atts['_type']) {
				case "string":
					break;
				case "date":
					$value = mysql2date(get_option('date_format'), $value);
					break;
			}
		}

		$content = "";
		/**
		 * Only show the <h1..x>-heading if label=yes and heading=h1|h2|h3|h4|h5|h6+
		 * Advantage: [cx_intro_information heading="no"] will drop the <h1..6>-tag!
		 */
		if ($show_label && strlen($heading)) {
			switch ($heading) {
				case "h1":
				case "h2":
				case "h3":
				case "h4":
				case "h5":
				case "h6":
					$content .= "<{$heading} class={$attr_quote}cx_{$heading} {$css_class}{$attr_quote}>{$section_name}</{$heading}>\n";
					break;
				case "span":
				case "div":
					$content .= "<{$heading} class={$attr_quote}{$css_class}{$attr_quote}>{$section_name}</{$heading}>\n";
					break;
				case "no":
					break;
			}
		}
		$container_tag = strtolower($atts['container']) == 'div' ? 'div' : 'span';
		if ($show_content) {
			if ($show_raw) $content .= "{$value}\n";
			else $content .= "<{$container_tag} class={$attr_quote}{$css_class}{$attr_quote}>{$value}</{$container_tag}>\n";
		}

		// if ( $section_key == "introInformation" ) $content .= '<!--more-->';

		return $content;
	}

	/**
	 * @param $atts array field (required): fieldname, separator (optional): for lists/checkboxes, type (optional): string|date
	 * @param $content
	 * @param $dataDictionary
	 *
	 * @return string
	 */
	public function vacancy_additional_field( $atts, $content, $dataDictionary ) : string {
		$additionalfieldssupport_status = Environment_Settings::getCxAdditionalFieldsSupportStatus();

		$fieldname = $atts['field'] ?? "";
		$fieldname = trim(strtolower($fieldname));
		$separator = $atts['separator'] ?? ", "; // checkboxes/multi-options are separated
		$fieldtype = $atts['type'] ?? "string"; // currently date or string

		if ($additionalfieldssupport_status == "off") {
			$content = "";
		} else if ( strlen($fieldname) == 0 ) {
			$extra_fields = $this->interfacer->db_get_additional_cx_fields();
			$content = "Shortcode error. ";
			if ( count($extra_fields) == 0) {
				$content .= "There are no correctly defined Additional Fields present in the Carerix System.";
			} else {
				$extra_fields = str_replace(WP_Constants::ADDITIONAL_FIELD_PREFIX, "", $extra_fields);
				$content .= "Empty or no fieldname supplied.<br>
							<br>
							Usage example: [{$atts['shortcode_name']} field=\"{$extra_fields[0]}\" separator=\", \"]<br>
							<br>
							The following additional fields are found: " . implode(", ", $extra_fields);
			}
			$content = "<div style='padding: 10px; border: red solid 2px;'>{$content}</div>";
		} else {
			$fieldname = preg_replace("/[^a-z0-9]+/", "_", $fieldname); // only allow a-z and digits. The rest is replaced by underscores.
			if ( strpos($fieldname, WP_Constants::ADDITIONAL_FIELD_PREFIX) !== 0 ) {
				$fieldname = WP_Constants::ADDITIONAL_FIELD_PREFIX . $fieldname;
			}

			/**
			 * First, try to fetch the value from the 'dataDictionary' . The dictionary is filled from the REST - API publication fetch.
			 * If that fails, it's a shortcode used on the front-end. For example in a theme template (do_shortcode()) or as user-input ([cx_extra...])
			 */
			$content = $dataDictionary[$fieldname] ?? get_post_meta(get_the_ID(), $fieldname) ?? "";

			if (is_array($content)) {
				$content = implode($separator, $content);
			}

			if ($fieldtype == "date") {
				$content = mysql2date( get_option( 'date_format' ), $content ) ?: "";
			}

		}
		return $content;
	}


	/**
	 * To manually add the GFJ snippet to the vacancy post page
	 *
	 * @param       $atts
	 * @param       $content
	 * @param array $dataDictionary
	 *
	 * @return string
	 */
	public function vacancy_gfj( $atts, $content, array $dataDictionary ) : string {
		if ( count($dataDictionary) ) return ""; // Bail out when used in WP Admin -> CX plugin -> Sources

		$template = new Utils_Template("fragments/frontend/googleforjobs_detail");

		$content = $template->with( get_the_content() )->display( true );
		return $content;
	}

	/**
	 * @param       $atts
	 * @param       $content
	 * @param array $dataDictionary
	 *
	 * @return string
	 */
	public function apply_until($atts, $content, array $dataDictionary ) : string {
		$date_publicationEnd = $dataDictionary["endDate"] ?? get_post_meta(get_the_ID(), "endDate", true) ?? "";

		if ( strlen($date_publicationEnd) ) {
			$date_publicationEnd = mysql2date("Y-m-d H:m:s", $date_publicationEnd . " -1 day");
		}
		$date_deadlineMaterial = $dataDictionary["deadlineMaterial"] ?? get_post_meta(get_the_ID(), "deadlineMaterial", true) ?? "";
		$date_reactionEndDate = $dataDictionary["reactionEndDate"] ?? get_post_meta(get_the_ID(), "reactionEndDate", true) ?? "";

		$dates = array();
		if ( !empty($date_publicationEnd) ) $dates[] = $date_publicationEnd; // publication end date from cx publication
		if ( !empty($date_deadlineMaterial) ) $dates[] = $date_deadlineMaterial; // closing date from cx publication
		if ( !empty($date_reactionEndDate) ) $dates[] = $date_reactionEndDate; // closing date from cx job order
		sort($dates);

		if ( count($dates) ) {
			$the_date = mysql2date( get_option( 'date_format' ), $dates[0] );
		} else {
			$the_date = "";
		}

		return $the_date;
	}


    public function rss_for_category() {
        global $post;
        if(    $this->interfacer->frontend_displayed_single() === FALSE
			&& $this->interfacer->frontend_displayed_page() === FALSE
            || $post->post_type == 'page')
		{
			return;
		}
        ob_start();
        
        $labels = Utils_Labels::get();
        $category = get_the_category();
        $category_name = '';
        $term_id = 0;

        if(isset($category[0])) {
            $term = $category[0];
            $term_id = $term->term_id;
            $category_name = $term->name ;
        }

        $feedgeneral = get_category_feed_link($term_id);

        $div = "<div class='rss'><a id='carerix_rss_link' target='_blank' rel='nofollow' href='{$feedgeneral}'>{$labels['CxPortal.message.field.rss']} $category_name</a></div>";
        echo $div;
        
        // $out = ob_get_contents();
        $out = ob_get_clean();
        return $out;
    }

    public function rss_for_all() {
        global $post;
        if(    $this->interfacer->frontend_displayed_single() === FALSE
			&& $this->interfacer->frontend_displayed_page() === FALSE
            || $post->post_type == 'page')
		{
			return;
		}
        ob_start();
        $labels = Utils_Labels::get();
        $categoryLanguage = $this->interfacer->get_default_site_language();
        $resource = 'resource_category_'. str_replace('.', '_', 'general.header.publications');
        $categoryKey = Environment_Settings::_setting_get($resource . '_' . $categoryLanguage);

        if(!$categoryKey) $categoryKey = $labels[$resource];
        $term = get_term_by('name', $categoryKey, 'category');

        if($term === false) {
            $terms = get_the_category();
            foreach($terms as $_term) {
                if($_term->category_parent === 0) {
                    $term = $_term;
                    break;
                }
            }
        }
        
        $url = get_category_feed_link(@$term->term_id);

        $div = "<div class='rss_all'><a id='carerix_rss_link_all' target='_blank' rel='nofollow' href='$url'>{$labels['CxPortal.message.field.rssAll']}</a></div>";
        echo $div;
        
        // $out = ob_get_contents();
        $out = ob_get_clean();
        return $out;
    }

    public function print_link() : string {
        global $post;
        if(    $this->interfacer->frontend_displayed_single() === FALSE
			&& $this->interfacer->frontend_displayed_page() === FALSE
            || $post->post_type == 'page')
		{
			return "";
		}
        $div = "<div class='print'><a id='carerix_print_link' rel='nofollow' href='#' onclick='window.print();  '>{$this->labels['general.button.print']}</a></div>";
        return $div;
    }

	/**
	 * Vacancy Number
	 * Internal CX fieldname: vacancyID
	 *
	 * @return false|string|void
	 */
    public function vacancy_number() {
        global $post;
        if(    $this->interfacer->frontend_displayed_single() === FALSE
			&& $this->interfacer->frontend_displayed_page() === FALSE
            || $post->post_type == 'page')
		{
			return;
		}

        $post_id = $post->ID;
        $interfacer = WP_Interfacer::getInstance();
        $props = $interfacer->entity_get_properties($post_id);
        $vacancyID = (string)$props['vacancyID'][0];
        
        $div = "<div id='carerix_vacancy_id'><strong>{$this->labels['general.header.jobID']}:</strong> #{$vacancyID} </div>";
        return $div;
    }

	/**
	 * Vacancy reference code or vacancy ID
	 * Internal CX fieldname: vacancyNo|vacancyID
	 *
	 * @param       $atts
	 * @param       $content
	 * @param array $dataDictionary
	 *
	 * @return string
	 */
    public function vacancy_code( $atts, $content, array $dataDictionary  ) : string {
	    $type = strtolower($atts['type'] ?? "code");

	    $content = "";
		switch ($type) {
			case "code":
				$content = $dataDictionary['vacancyNo'] ?? get_post_meta(get_the_ID(), 'vacancyNo', true) ?? "";
				break;
			case "id":
				$content = $dataDictionary['vacancyID'] ?? get_post_meta(get_the_ID(), 'vacancyID', true) ?? "";
				break;
		}
	    return $content;
    }

	/**
	 * Medium code of the Carerix publication
	 * Internal CX fieldname: medium
	 *
	 * @param $atts
	 * @param $content
	 * @param array $dataDictionary The value is stored in the dataDictionary if used in a template of "CX Plugin->Sources". If shortcode is used on front-end the dataDictionary array is empty
	 *
	 * @return string
	 */
    public function vacancy_medium( $atts, $content, $dataDictionary  ) : string {
	    $type = strtolower($atts['type'] ?? "code");

		switch ($type) {
			case "code":
//				$content = get_post_meta(get_the_ID(), 'medium', true);
				$content = $dataDictionary['medium'] ?? get_post_meta(get_the_ID(), 'medium', true) ?? "";
				break;
			default:
				$content = "";
		}
	    return $content;
    }

	/**
	 * Publicaton ID
	 * Internal CX fieldname: guid
	 *
	 * @param $atts
	 * @param $content
	 * @param array $dataDictionary The value is stored in the dataDictionary if used in a template of "CX Plugin->Sources". If shortcode is used on front-end the dataDictionary array is empty
	 *
	 * @return string
	 */
    public function publication_id($atts, $content, array $dataDictionary  ) : string {
	    $content = $dataDictionary['guid'] ?? get_post_meta(get_the_ID(), 'guid', true) ?? "";
	    return $content;
    }

	/**
	 * Workplace type
	 * Internal CX fieldname: workplaceType, workplaceTypeClassification
	 *
	 * @param       $atts
	 * @param       $content
	 * @param array $dataDictionary
	 *
	 * @return string
	 */
    public function workplaceType( $atts, $content, array $dataDictionary  ) : string {
		$workplaceTypeClassification = $dataDictionary['workplaceTypeClassification'] ?? get_post_meta(get_the_ID(), 'workplaceTypeClassification', true) ?? ""; // e.g. WorkplaceHybridTag
	    $classificationText = $this->labels['CxTags.workplaceType.' . $workplaceTypeClassification] ?? "";

	    return $classificationText;
    }

	/**
	 * Salary range (
	 * Internal CX fieldnames: salaryCurrency, minSalary, maxSalary, salaryperiod
	 *
	 * @param $atts
	 *
	 * @return string
	 */
    public function salary( $atts, $content, $dataDictionary  ) : string {
	    $atts = shortcode_atts( array(
		    'raw' => 'no', // raw mode will leave out the formatting
		    'show' => 'csu' // c=currency, s=salary (range), u=unit
	    ), $atts );

	    $show_raw = $this->yesNoToBool(trim($atts['raw']), false);
		$show_elements = strtolower(trim($atts['show']));
		$show_currency = strpos($show_elements, "c") !== false;
		$show_salary = strpos($show_elements, "s") !== false;
		$show_unit = strpos($show_elements, "u") !== false;

	    $isSalaryPublic = $dataDictionary['isSalaryPublic'] ?? get_post_meta(get_the_ID(), 'isSalaryPublic', true) ?? false;

	    if (!$isSalaryPublic) return "";

	    $currency_id = $dataDictionary['salaryCurrency'] ?? get_post_meta(get_the_ID(), 'salaryCurrency', true) ?? ""; // EUR, USD

	    switch ($currency_id) {
		    case "EUR":
			    $currency_symbol = "€";
				break;
		    case "USD":
			    $currency_symbol = "$";
				break;
		    default:
			    $currency_symbol = "";
			    $show_currency = false;
		}
		if ($show_raw) $currency_symbol = $currency_id;

	    $salaryPeriodClassification = $dataDictionary['salaryPeriodClassification'] ?? get_post_meta(get_the_ID(), 'salaryPeriodClassification', true) ?? ""; // E.g. HourTag, DayTag, WeekTag, FourWeeksTag, MonthTag, YearTag

	    switch ($salaryPeriodClassification) {
		    case "hourtag":
			    $unitText = $this->labels['general.message.field.hourTag'];
			    break;
		    case "daytag":
			    $unitText = $this->labels['general.message.field.dayTag'];
			    break;
		    case "weektag":
			    $unitText = $this->labels['general.message.field.weekTag'];
			    break;
		    case "fourweekstag":
			    $unitText = $this->labels['general.message.field.fourWeeksTag'];
			    break;
		    case "monthtag":
			    $unitText = $this->labels['general.message.field.monthTag'];
			    break;
		    case "yeartag":
			    $unitText = $this->labels['general.message.field.yearTag'];
			    break;
		    default:
			    // Unrecognized classification
			    $unitText = "";
			    $show_unit = false;
	    }
	    if ($show_raw) $unitText = $salaryPeriodClassification;


	    $minSalary = (double) ($dataDictionary['minSalary'] ?? get_post_meta(get_the_ID(), 'minSalary', true) ?? "");
	    $maxSalary = (double) ($dataDictionary['maxSalary'] ?? get_post_meta(get_the_ID(), 'maxSalary', true) ?? "");

	    $content = array();
	    if ($show_currency) $content[] = $currency_symbol;
	    if ($minSalary > 0 && $maxSalary > $minSalary) {
		    if ($show_salary) $content[] = "{$minSalary}-{$maxSalary}";
	    } else {
		    $singleSalary = $minSalary + $maxSalary; // One of the operands is zero
		    if ($show_salary) $content[] = $singleSalary;
	    }
	    if ( $show_unit ) $content[] = $unitText;

		$content = implode(" ", $content);

		return $content;
    }

    public function social_icons() : string {
        global $post;
        if(    $this->interfacer->frontend_displayed_single() === FALSE
			&& $this->interfacer->frontend_displayed_page() === FALSE
            || $post->post_type == 'page')
		{
			return "";
		}
        $div = '<div class="social_icons">
						<div id="cx_socials">
							<span class="st_facebook"></span><span class="st_twitter"></span>					
                            <span class="st_linkedin"></span>
						</div>
                        <script type="text/javascript">var switchTo5x=true;</script>
						<script type="text/javascript" src="https://ws.sharethis.com/button/buttons.js"></script>
                        <script type="text/javascript">stLight.options({publisher: "9b61bbc7-5b60-40a2-9ea9-36c8b84e5406", doNotHash: false, doNotCopy: false, hashAddressBar: false});</script>
						
                    </div>'; // <span class="st_email"></span>
        return $div;
    }

	public function apply_button() {
		global $post;
		if(    $this->interfacer->frontend_displayed_single() === FALSE
			&& $this->interfacer->frontend_displayed_page() === FALSE
			|| $post->post_type == 'page')
		{
			return;
		}
		$post_permalink = $this->interfacer->get_post_permalink(false, 'apply=true');
		$label = (isset($_GET['apply']) && $_GET['apply'] == 'true') ? $this->labels['general.button.applicationSubmit'] : $this->labels['general.button.apply'];
		return "<div><a id='apply_button' class='cx2_vacancydetails-apply' href='$post_permalink'>{$label}</a></div>";
	}

    public function function_group_legacy() {
        global $post;
        if(    $this->interfacer->frontend_displayed_single() === FALSE
			&& $this->interfacer->frontend_displayed_page() === FALSE
            || $post->post_type == 'page')
		{
			return;
		}
        ob_start();
        
        $post_id = $post->ID;

        $interfacer = WP_Interfacer::getInstance();
        $props = $interfacer->entity_get_properties($post_id);
        $pubKey = '_carerix_job';
        if($this->interfacer->wpml_is_active()) {
            $pubKey .= '_'.$this->interfacer->get_default_site_language();
        }
        
        $pubID = (string)$props[$pubKey][0];
        
        if($pubID) {
            $params = array('show' => array('toVacancy', 'toVacancy.toFunctionLevel1.value','toVacancy.toFunctionLevel2.value','toVacancy.toFunctionLevel3.value', 'publicationID') );

            $publication = CRPublication::find($pubID, $params);
            $labels = Utils_Labels::get();

            $functionGroup = implode(' / ', array_filter(array(
                            $publication->toVacancy->toFunctionLevel1->value,
                            $publication->toVacancy->toFunctionLevel2->value,
                            $publication->toVacancy->toFunctionLevel3->value
                        )));
            $div = '';
            if($functionGroup) $div = "<div id='carerix_function_group'><strong>{$labels['CxPortal.message.field.functionGroup']}:</strong> $functionGroup</div>";
            echo $div;
        }
        
        // $out = ob_get_contents();
        $out = ob_get_clean();
        return $out;
    }



    
//    static function get_source_data() {
//        global $post;
//        if(    $this->interfacer->frontend_displayed_single() === FALSE
//			&& $this->interfacer->frontend_displayed_page() === FALSE
//            || $post->post_type == 'page')
//		{
//			return;
//		}
//
//        $post_id = $post->ID;
//        $interfacer = WP_Interfacer::getInstance();
//        $props = $interfacer->entity_get_properties($post_id);
//        $source_id = (string)$props['source'][0];
//        $source = Table_Source::get();
//        $source_data = $source->getByID($source_id);
//        $source_data['settings'] = unserialize($source_data['settings']);
//        return $source_data;
//    }

	/**
	 * @return array
	 */
	public function getLabels( $categoryLanguage = false )
	{
		if(isset($this->dataDictionary['medium'])) {
			$_l = explode('-', $this->dataDictionary['medium']);
			if(isset($_l[1])) {
				$language = trim($_l[1]);

				$language .= '-' . strtoupper($language);
				$labels = Utils_Labels::get($language);

			} else {
				$labels = Utils_Labels::get();
			}
		} else {
			$labels = Utils_Labels::get($categoryLanguage);
		}

		return $labels;
	}

	/**
	 * @deprecated since v3.1.0
	 * @return string[]
	 */
	public function getContentSections() {
		$contentSections = array(
			'introInformation',
			'companyInformation',
			'vacancyInformation',
			'requirementsInformation',
			'offerInformation',
			'endDate',
			'functionContactInformation',
			'applicationContactInformation',
		);

		return $contentSections;
	}


	/**
	 * @param $section
	 *
	 * @return string
	 */
	public function getSectionLabelKey( $section ): string {
		$labelKeysForSection = array(
			'rawTitle' => 'general.message.field.jobTitle',
			'introInformation' => 'CxPortal.message.field.introduction',
			'companyInformation' => 'CxPortal.message.field.organization',
			'vacancyInformation' => 'CxPortal.message.field.function',
			'requirementsInformation' => 'CxPortal.message.field.requirements',
			'offerInformation' => 'CxPortal.message.field.offer',
			'functionContactInformation' => 'CxPortal.message.field.information',
			'applicationContactInformation' => 'CxPortal.message.field.application',
			'company' => 'CxPortal.message.field.organization',
			'endDate' => 'CxPortal.message.field.closingDate',
			'hoursPerWeek' => 'CxPortal.message.field.weekHours',
			'businessLine' => 'CxPortal.message.field.businessLine', // area of expertise / vakgebied
			'branche0' => 'CxPortal.message.field.sector',
			'branche1' => 'CxPortal.message.field.industry',
			'function0' => 'CxPortal.message.field.singleFunctionGroup',
			'function1' => 'CxPortal.message.field.function',
			WP_Constants::TAXONOMY_JOBGROUP => 'CxPortal.message.field.jobGroup',
			'contractType' => 'CxPortal.message.field.contractType',
			'agency' => 'CxPortal.message.field.office',
			'country' => 'general.message.field.country',
			'region' => 'general.message.field.region',
			'workLocation' => 'CxPortal.message.field.workLocation',
			'postalCode' => 'CxPortal.message.field.workPostalCode',
			'toWorkLevelNode' => 'CxPortal.message.field.professionalLevel',
			WP_Constants::TAXONOMY_EDUCATION => 'general.header.education',
			'procedure' => 'CxPortal.message.field.procedure',
		);

		return $labelKeysForSection[$section] ?? "";
	}

	private function yesNoToBool(string $value, bool $default = true)
	{
		$value = trim(strtolower($value));
		switch ($value) {
			case "yes":
			case "1":
			case "true":
			case "on":
			case "show":
				$return_val = true;
				break;
			case "no":
			case "0":
			case "false":
			case "off":
			case "hide":
				$return_val = false;
				break;
			default:
				$return_val = $default;
		}

		return $return_val;
	}

}

