<?php /** @noinspection PhpUndefinedConstantInspection */

namespace Carerix\Cookie;

/**
 * Sets/reads/deletes cookies
 * Supports storing arrays and objects too
 *
 * @author Ruben <ruben@carerix.com>
 */
class Cookie
{
	const EXPIRY_SECONDS = 30*24*3600;
	const PREFIX = "CXCOOKIE";

	/**
	 * @return string
	 */

	public static function getHashKey( string $ownHashKey = null ) : string {
		if (isset($ownHashKey)) {
			$hashkey = $ownHashKey;
		} else {
			$alt_key = "Carerix-" . ($_SERVER['REMOTE_ADDR'] ?? "") . "-" . ($_SERVER['HTTP_USER_AGENT'] ?? "");
			$hashkey = $_COOKIE['PHPSESSID'] ?? md5( $alt_key );
		}

		return $hashkey;
	}

	public static function setCookie($name, $value, string $ownHashkey = null, $expiry_seconds = self::EXPIRY_SECONDS ) {
		set_transient( self::__constructTransientName($name, $ownHashkey), $value, $expiry_seconds );

/*
		$secure = ( 'https' === parse_url( wp_login_url(), PHP_URL_SCHEME ) );
		@setcookie($name, json_encode($value), time() + $expiry_seconds, COOKIEPATH, COOKIE_DOMAIN, $secure);
		if ( SITECOOKIEPATH !== COOKIEPATH ) {
			@setcookie($name, json_encode($value), time() + $expiry_seconds, SITECOOKIEPATH, COOKIE_DOMAIN, $secure);
		}
*/
	}

	public static function getCookie( string $name, $default_value = "", string $ownHashkey = null) {
		$ret_value = get_transient( self::__constructTransientName($name, $ownHashkey) ) ?: $default_value;

/*		$ret_value = $default_value;
		if (isset($_COOKIE[$name])) $ret_value = $_COOKIE[$name];
		$mixed_val = json_decode(stripslashes($ret_value), $as_associative_array);
		if (isset($mixed_val)) $ret_value = $mixed_val;
*/

		return $ret_value;
	}

	public static function deleteCookie( string $name, string $ownHashkey = null ) {
		delete_transient( self::__constructTransientName($name, $ownHashkey) );

/*		$secure = ( 'https' === parse_url( wp_login_url(), PHP_URL_SCHEME ) );
		unset($_COOKIE[$name]);
		@setcookie($name, "",  time()-7200, COOKIEPATH, COOKIE_DOMAIN, $secure);
		if ( SITECOOKIEPATH !== COOKIEPATH ) {
			@setcookie($name, "",  time()-7200, SITECOOKIEPATH, COOKIE_DOMAIN, $secure);
		}*/

	}

	private static function __constructTransientName( string $name, string $ownHashkey = null ): string {
		return self::PREFIX . "-" . self::getHashKey( $ownHashkey ) . "-" . $name;
	}
}